%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear, clc, close all
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Question 1: get data
[filename,pathname]=uigetfile('*.xls');
[data,textdata,raw] = xlsread(filename,'Monthly');

%Convert dates into a serial number
date=datenum(textdata(4:end,1),'dd/mm/yyyy');

%% Question 2: compute log excess returns for the risky assets 
%Log monthly risk free rate
lrf_m  = log(1+(lag(data(:,2))/(100*12)));
lrf_m(1,1)=NaN;
%Log monthly exc return German Long term bond
ly_10_m   = (log(1+(data(:,1))/(100*12)));
dur = 12*((1-(1+(data(:,1)/(100))).^(-10)))./(1-(1+(data(:,1)/(100))).^(-1));
lret_b_10_m =(lag(dur).*lag(ly_10_m)-(lag(dur)-1).*ly_10_m);
exlret_b_10 = lret_b_10_m-lrf_m;


%Log monthly exc return German stock market
dy_ger_m = data(:,4)/(100*12);
ldy_ger = log(data(:,4)); % annualized
p_ger  = data(:,3);
lret_ger_m = log((p_ger./lag(p_ger))+dy_ger_m);
lret_ger_m(1,1)=NaN;
exlret_ger = lret_ger_m - lrf_m;

%FOREX
EUvsDOL=data(:,10);
STRvsDOL=data(:,9);
STRvsEU=STRvsDOL.*EUvsDOL;
r_EUvsDOL  = log((data(:,10))./lag(data(:,10)));
r_EUvsDOL(1,1)=NaN;
r_STRvsDOL = log((data(:,9))./lag(data(:,9)));
r_STRvsDOL(1,1)=NaN;
r_STRvsEU  = r_STRvsDOL + r_EUvsDOL;
r_STRvsEU(1,1)=NaN;
%Log monthly exc return US stock market
dy_us_m = data(:,6)/(100*12);
ldy_us = log(data(:,6)); % annualized
p_us = data(:,5);
lret_us_dol = log((p_us./lag(p_us))+dy_us_m);
lret_us_dol(1,1)=NaN;
lret_us_m = lret_us_dol + r_EUvsDOL;
exlret_us = lret_us_m - lrf_m;

%Log monthly exc return UK stock market
dy_uk_m = data(:,8)/(100*12);
ldy_uk = log(data(:,8)); % annualized
p_uk = data(:,7);
lret_uk_str = log((p_uk./lag(p_uk))+dy_uk_m);
lret_uk_str(1,1)=NaN;
lret_uk_m = lret_uk_str + r_STRvsEU;
exlret_uk = lret_uk_m - lrf_m;

%% Question 3: Plotting Historical Performance 

% Defining Sample 1 = 01/1978:12/2003
s_start1 = '31/01/1978';
s_end1   = '31/12/2003';
date_find1=datenum([s_start1; s_end1],'dd/mm/yyyy'); % Lookup row number
ss1=datefind(date_find1(1,1),date); % Sample Start (First Raw)
se1=datefind(date_find1(2,1),date); % Sample End (Last Raw)
ObsSample1 = se1-ss1+1; % Number of observations within Sample1

% Historical Excess Returns Matrix in Sample 1
EX_R1 = [exlret_b_10(ss1:se1) exlret_ger(ss1:se1) exlret_uk(ss1:se1) exlret_us(ss1:se1)];
R1 = [lret_b_10_m(ss1:se1) lret_ger_m(ss1:se1) lret_uk_m(ss1:se1) lret_us_m(ss1:se1)];
% Plot performances of different asset classes 
Perf = cumsum(EX_R1);

figure(1);
plot(Perf);
title('Assets Excess Returns 01/1978:12/2003','fontname','Garamond','fontsize',14);
index=1:60:(ObsSample1);
set(gca,'fontname','garamond','fontsize',10);
set(gca,'xtick',index);
set(gca,'xticklabel','Jan1978|Jan1983|Jan1988|Jan1993|Jan1997|Jan2002');
set(gca,'xlim',[1 (ObsSample1)]);
grid;
ylabel('Returns');
xlabel('Date');
h=legend('German LT Bond', 'German Equity','US Equity','UK Equity',0);
%% Question 4: Weights in the Market Portfolio at 12/2003 

% Defining the sample period 2 30/01/2004 : 31/12/2007
s_start2 = '30/01/2004';
s_end2   = '31/12/2007';
date_find2=datenum([s_start2; s_end2],'dd/mm/yyyy');
ss2=datefind(date_find2(1,1),date);
se2=datefind(date_find2(2,1),date);
ObsSample2 = se2-ss2+1;

% Creating Sample 2 Excess Returns and Returns Matrix
EX_R2 = [exlret_b_10(ss2:se2) exlret_ger(ss2:se2) exlret_uk(ss2:se2) exlret_us(ss2:se2)];
R2 = [lret_b_10_m(ss2:se2) lret_ger_m(ss2:se2) lret_uk_m(ss2:se2) lret_us_m(ss2:se2)];

% Compute unconditional means and VarCov

MuHat1 = mean(EX_R1); % Sample mean **
muR = ObsSample2*MuHat1';
%muR = ObsSample2*mean(EX_R1)';
SigmaR = ObsSample2*cov(R1);
% WEIGHTS (see solutions on Corielli's lecture notes, CH 8 pag. 56)
wMP = ((SigmaR^(-1))*muR)./(ones(4,1)'*(SigmaR^(-1)*muR));



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% STANDARD VS BOOTSTRAPPED PORTFOLIO - NEW PART!
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Defining Sample 3 for Rolling Analysis

s_start3 = '31/01/1978';
s_end3 = '28/12/2012';
window = 60; % Size in month of the rolling window if you want to doublecheck 
% the algorithm set the windows at 312 obs and comment the for cycle, i.e. 
% the size of sample1 and then compare the portfolio weights
date_find3=datenum([s_start3; s_end3],'dd/mm/yyyy'); % Lookup row number
ss3=datefind(date_find3(1,1),date);
se3=datefind(date_find3(2,1),date);
LastRoll = se3-ss3+1-window; % Last Rolling Window

counter = 0;

for j = 1 : 12 : LastRoll

    counter = counter + 1;
    fprintf(1,'Calculating Rolling Bootstrapped Portfolio: Year ');
    fprintf(1,'%i',counter);
    
    
    ssRolling = ss3 + j -1;  % Rolling Sample Start (First Raw)
    seRolling= ssRolling + window - 1; % Sample End (Last Raw)
    ObsSampleRolling = seRolling-ssRolling+1; % Number of observations within RollingSample

    % Historical Excess Returns Matrix in RollingSample 
    EX_RRolling = [exlret_b_10(ssRolling:seRolling) exlret_ger(ssRolling:seRolling) exlret_uk(ssRolling:seRolling) exlret_us(ssRolling:seRolling)];
    RRolling = [lret_b_10_m(ssRolling:seRolling) lret_ger_m(ssRolling:seRolling) lret_uk_m(ssRolling:seRolling) lret_us_m(ssRolling:seRolling)];

    % Standard Markowitz Portfolio ========================================

    MuHatRolling = mean(EX_RRolling); % Sample mean **
    muR = MuHatRolling';
    SigmaR = cov(RRolling);
    % WEIGHTS (see solutions on Corielli's lecture notes, CH 8 pag. 56)
    wMPRolling(:,counter) = ((SigmaR^(-1))*muR)./(ones(4,1)'*(SigmaR^(-1)*muR));
    
    % Bootstrapped Portfolio ==============================================
    
    m = 10000; % Number of Boostrap Draws
    wMP_Boot = NaN(4,m);

    % Generate the Residuals Matrix V

    V = EX_RRolling-(ones(ObsSampleRolling,1)*MuHatRolling); %$$

    for i = 1:m

        % Resample the residuals with replacement

        % if you want to check the algorithm change randsample(1:ObsSampleRolling,ObsSampleRolling,'true')...
        % with 1:1:ObsSampleRolling; and compare the bootstrap mean with the
        % simple one
    
        RawsSampled = randsample(1:ObsSampleRolling,ObsSampleRolling,'true');
        V_Boot = V(RawsSampled,:); %$$
        EX_R_Boot = V_Boot + ones(ObsSampleRolling,1)*MuHatRolling; %$$
        R_Boot = RRolling(RawsSampled,:); % Resample the returns
        %EX_R_Boot = EX_RRolling(RawsSampled,:); %$$
    
        MuHat_Boot = mean(EX_R_Boot); % Sample mean **
        muR_Boot = MuHat_Boot';
        SigmaR_Boot = cov(R_Boot); 
        % WEIGHTS (see solutions on Corielli's lecture notes, CH 8 pag. 56)
        wMP_Boot(:,i) = ((SigmaR_Boot^(-1))*muR_Boot)./(ones(4,1)'*(SigmaR_Boot^(-1)*muR_Boot));
        %clear RawsSamples V_Boot EX_R_Boot MuHat_Boot muR_Boot SigmaR_Boot
    end


    wMP_Bootstrap_Mean(:,counter) = mean(wMP_Boot,2);
    wMP_Bootstrap_Median(:,counter) = median(wMP_Boot,2);

    clc
end

fprintf('\n')

% Plot comparison

Label1 = cellstr('German LT Bond'); % Generating a vector of label
Label2 = cellstr('German Equity');
Label3 = cellstr('US Equity');
Label4 = cellstr('UK Equity');
LabelVector = vertcat(Label1,Label2,Label3,Label4);

figure(7)
for jj = 1:4
    
    subplot(2,2,jj);
    plot(wMPRolling(jj,:),'Linewidth',2.5); hold on;
    plot(wMP_Bootstrap_Median(jj,:),'Color','r','Linewidth',2.5);
    title(LabelVector(jj));
    set(gca,'xtick',1:5:counter);
    set(gca,'xlim',[0 counter]);
    set(gca,'xticklabel','1/1983|1/1988|1/1993|1/1998|1/2003|1/2008|1/2013');
    grid on;
    ylabel('Portfolio Weight');
    legend('Standard','Bootstrap')
    
end    

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
%% Question 5: Plot Weights and Performance
% Plot Weights
figure(2)
bar(wMP)
title('Weights of Assets in the Mkt Portfolio 01/1978:12/2003','fontname','Garamond','fontsize',14);
set(gca,'YGrid','on')
set(gca,'fontname','garamond','fontsize',10);
set(gca,'xticklabel','German LT Bond|German Equity|US Equity|UK Equity');
ylabel('Percentage Weight');
colormap summer
% Performance
PerfEX_R2 = cumsum(EX_R2);
Rport = EX_R2*wMP;
PerfPort1 = cumsum(Rport);

% mio

Rport_Bootstrap = EX_R2*wMP_Bootstrap_Median;
PerfPort1_Bootstrap = cumsum(Rport_Bootstrap);

% mio

% Plot Perf
figure(3)
plot(PerfEX_R2); hold on; plot(PerfPort1,'k','LineWidth',2.7,'LineStyle','--')
title('Assets Excess Returns and Portfolio Performance','fontname','Garamond','fontsize',14);
index=1:12:(ObsSample2);
set(gca,'fontname','garamond','fontsize',10);
set(gca,'xtick',index);
set(gca,'xticklabel','Jan2004|Jan2005|Jan2006|Jan2007');
set(gca,'xlim',[1 (ObsSample2)]);
grid;
ylabel('Returns');
xlabel('Date');
h=legend('German LT Bond', 'German Equity','US Equity','UK Equity','Portfolio',0);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Question 6: Weights in the Market Portfolio at 01/2004 12/2007

muR_end = ObsSample2*mean(EX_R2)';
SigmaR_end = ObsSample2*cov(R2);
% WEIGHTS (see solutions on Corielli's lecture notes, CH 8 pag. 56)
wMKT_end = ((SigmaR_end^(-1))*muR_end)./(ones(4,1)'*(SigmaR_end^(-1)*muR_end));
% Plot Weights
figure(4)
bar(wMKT_end)
title('Weights of Assets in the Mkt Portfolio 12/2005:12/2007','fontname','Garamond','fontsize',14);
set(gca,'YGrid','on')
set(gca,'fontname','garamond','fontsize',10);
set(gca,'xticklabel','German LT Bond|German Equity|US Equity|UK Equity');
ylabel('Percentage Weight');
colormap summer
%% Question 7: What if you had used Black and Litterman? (Corielli pag. 91)
%Assumptions, see solutions
wMP_i = [0.25 0.25 0.25 0.25]';
lambda = 2.5;
% Implied expecte excess returns
muMP_i = SigmaR*wMP_i./lambda;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Inputs, see solutions

P = [ 0 1 0 0;
      0 0 1 0;
      0 0 0 1];

%lower bound 95% = 0, upper bound 95% = 0.8
V = [0.4 0.4 0.4]';

Gamma =[0.04  0     0    ;
        0     0.04  0    ;
        0     0     0.04];

tau = 0.3;

%B&L weights; see Corielli and solutions
K = tau*SigmaR*P'*(tau*P*SigmaR*P' + Gamma)^(-1);
sBL = lambda*((SigmaR^(-1))*(muMP_i) + (SigmaR^(-1))*K*(V-P*muMP_i));
wBL = sBL./(ones(4,1)'*sBL);

% Performance
RportBL = EX_R2*wBL;
PerfPort2 = cumsum(RportBL);

% Plot, BL Portfolio Weights
figure(5)
bar(wBL)
title('Weights of Assets in the BL Portfolio','fontname','Garamond','fontsize',14);
set(gca,'YGrid','on')
set(gca,'fontname','garamond','fontsize',10);
set(gca,'xticklabel','German LT Bond|German Equity|US Equity|UK Equity');
ylabel('Percentage Weight');
colormap summer
% Plot, BL Portfolio Perf
figure(6)
plot(PerfEX_R2); hold on; plot(PerfPort2,'k','LineWidth',2.7,'LineStyle','--')
title('Assets Excess Returns and Portfolio Performance','fontname','Garamond','fontsize',14);
index=1:12:(ObsSample2);
set(gca,'fontname','garamond','fontsize',10);
set(gca,'xtick',index);
set(gca,'xticklabel','Jan2004|Jan2005|Jan2006|Jan2007');
set(gca,'xlim',[1 (ObsSample2)]);
grid;
ylabel('Returns');
xlabel('Date');
h=legend('German LT Bond', 'German Equity','US Equity','UK Equity','Portfolio',0);