%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear, clc, close all
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Question 1: get data
[filename,pathname]=uigetfile('*.xls');
[data,textdata,raw] = xlsread(filename,'Monthly');

%Convert dates into a serial number
date=datenum(textdata(4:end,1),'dd/mm/yyyy');

%% Question 2: compute log excess returns for the risky assets 
%Log monthly risk free rate
lrf_m  = log(1+(lag(data(:,2))/(100*12)));
lrf_m(1,1)=NaN;
%Log monthly exc return German Long term bond
ly_10_m   = (log(1+(data(:,1))/(100*12)));
dur = 12*((1-(1+(data(:,1)/(100))).^(-10)))./(1-(1+(data(:,1)/(100))).^(-1));
lret_b_10_m =(lag(dur).*lag(ly_10_m)-(lag(dur)-1).*ly_10_m);
exlret_b_10 = lret_b_10_m-lrf_m;


%Log monthly exc return German stock market
dy_ger_m = data(:,4)/(100*12);
ldy_ger = log(data(:,4)); % annualized
p_ger  = data(:,3);
lret_ger_m = log((p_ger./lag(p_ger))+dy_ger_m);
lret_ger_m(1,1)=NaN;
exlret_ger = lret_ger_m - lrf_m;

%FOREX
EUvsDOL=data(:,10);
STRvsDOL=data(:,9);
STRvsEU=STRvsDOL.*EUvsDOL;
r_EUvsDOL  = log((data(:,10))./lag(data(:,10)));
r_EUvsDOL(1,1)=NaN;
r_STRvsDOL = log((data(:,9))./lag(data(:,9)));
r_STRvsDOL(1,1)=NaN;
r_STRvsEU  = r_STRvsDOL + r_EUvsDOL;
r_STRvsEU(1,1)=NaN;
%Log monthly exc return US stock market
dy_us_m = data(:,6)/(100*12);
ldy_us = log(data(:,6)); % annualized
p_us = data(:,5);
lret_us_dol = log((p_us./lag(p_us))+dy_us_m);
lret_us_dol(1,1)=NaN;
lret_us_m = lret_us_dol + r_EUvsDOL;
exlret_us = lret_us_m - lrf_m;

%Log monthly exc return UK stock market
dy_uk_m = data(:,8)/(100*12);
ldy_uk = log(data(:,8)); % annualized
p_uk = data(:,7);
lret_uk_str = log((p_uk./lag(p_uk))+dy_uk_m);
lret_uk_str(1,1)=NaN;
lret_uk_m = lret_uk_str + r_STRvsEU;
exlret_uk = lret_uk_m - lrf_m;

%% Question 3: Plotting Historical Performance 

% Defining Sample 1 = 01/1978:12/2003
s_start1 = '31/01/1978';
s_end1   = '28/12/2012';
date_find1=datenum([s_start1; s_end1],'dd/mm/yyyy'); % Lookup row number
ss1=datefind(date_find1(1,1),date); % Sample Start (First Raw)
se1=datefind(date_find1(2,1),date); % Sample End (Last Raw)
ObsSample1 = se1-ss1+1; % Number of observations within Sample1

% Historical Excess Returns Matrix in Sample 1
EX_R1 = [exlret_b_10(ss1:se1) exlret_ger(ss1:se1) exlret_uk(ss1:se1) exlret_us(ss1:se1)];
R1 = [lret_b_10_m(ss1:se1) lret_ger_m(ss1:se1) lret_uk_m(ss1:se1) lret_us_m(ss1:se1)];
% Plot performances of different asset classes 
Perf = cumsum(EX_R1);

figure(1);
plot(Perf);
title('Assets Excess Returns 01/1978:12/2012','fontname','Garamond','fontsize',14);
index=1:60:(ObsSample1);
set(gca,'fontname','garamond','fontsize',10);
set(gca,'xtick',index);
set(gca,'xticklabel','Jan1978|Jan1983|Jan1988|Jan1993|Jan1997|Jan2002|Jan2007|Jan2013');
set(gca,'xlim',[1 (ObsSample1)]);
grid;
ylabel('Returns');
xlabel('Date');
h=legend('German LT Bond', 'German Equity','US Equity','UK Equity',0);

%% Question 4 Compare Log Returns vs. Log-Linearized Returns

pd_ger = 1./dy_ger_m; % price dividend ratio
rho_ger = mean(pd_ger)/(1+mean(pd_ger)); % Rho
d_growth_ger = log((p_ger(2:end,1).*dy_ger_m(2:end,1))./(p_ger(1:end-1,1).*dy_ger_m(1:end-1,1))); % Dividend Growth
% Log_lin_ret
LogLinRet_ger = rho_ger*(-ldy_ger(2:end,1))+ d_growth_ger - (-ldy_ger(1:end-1,1));  
% Comparison Matrix
Comparison = [lret_ger_m(2:end,1),LogLinRet_ger];
CumulatedComparison = cumsum(Comparison);
obs_comparison = size(Comparison,1);

figure(2);

plot(Comparison(:,1),'Color','b','Linewidth',1.2);hold on;
plot(Comparison(:,2),'Color','r','Linewidth',1,'LineStyle','--');
title('Log Returns vs. Log-Linearized Returns','fontname','Garamond','fontsize',14);
index=1:60:obs_comparison;
set(gca,'fontname','garamond','fontsize',10);
set(gca,'xtick',index);
set(gca,'xticklabel','Jan1978|Jan1983|Jan1988|Jan1993|Jan1997|Jan2002|Jan2007|Jan2013');
set(gca,'xlim',[1 obs_comparison]);
grid;
ylabel('Returns');
xlabel('Date');
h=legend('German Log-Returns', 'German Log-Linearized Rets',0);

figure(3);

plot(CumulatedComparison(:,1),'Color','b','Linewidth',1.2);hold on;
plot(CumulatedComparison(:,2),'Color','r','Linewidth',1,'LineStyle','--');
title('Cumulated Log Returns vs. Log-Linearized Returns','fontname','Garamond','fontsize',14);
index=1:60:obs_comparison;
set(gca,'fontname','garamond','fontsize',10);
set(gca,'xtick',index);
set(gca,'xticklabel','Jan1978|Jan1983|Jan1988|Jan1993|Jan1997|Jan2002|Jan2007|Jan2013');
set(gca,'xlim',[1 obs_comparison]);
grid;
ylabel('Returns');
xlabel('Date');
h=legend('German Cumulated Log-Returns', 'German Cumulated Log-Linearized Rets',0);



