%%-------------------------------------------------------------------------
% Main code for chapter 4: the CER model
%--------------------------------------------------------------------------

% -------------- Room saving ----------------------------------------------

close all; clear all; clc; pause(0.01), randn('seed',sum(clock)), rand('seed',sum(clock)), warning off
TSTART = tic;
% -------------- Load folders ---------------------------------------------

addpath([pwd '\Input\']);
addpath([pwd '\Utilities\']);
addpath([pwd '\Output\']);

% -------------- Import data ----------------------------------------------

[data,textdata,raw] = xlsread('STOCKINT2013.xls','Monthly');
%[filename,pathname]=uigetfile('*.xls');
%[data,textdata,raw] = xlsread(filename,'Monthly');

%Convert dates into a serial number
date=datenum(textdata(4:end,1),'dd/mm/yyyy');
% BUILDING RETURNS 
% --------------  German risk-free please note dating ---------
GER.RiskFree  = log(1+(lag(data(:,2))/(100*12)));
GER.RiskFree(1,1)=NaN;
%  German Long term bond YTM and period holding returns
GER.LTBond.Yield   = (log(1+(data(:,1))/(100*12)));
GER.LTBond.Duration = 12*((1-(1+(data(:,1)/(100))).^(-10)))./(1-(1+(data(:,1)/(100))).^(-1));
GER.LTBond.Ret   = (lag(GER.LTBond.Duration,1,NaN).*lag(GER.LTBond.Yield,1,NaN)-(lag(GER.LTBond.Duration,1,NaN)-1).*GER.LTBond.Yield);
GER.LTBond.ExRet     = GER.LTBond.Ret - GER.RiskFree;

% -------------- German Stocks Monthly excess returns  ------------------
GER.Stock.DY     = data(:,4)/(100*12);                                                          % Monthly dividend yield
GER.Stock.dy     = log(GER.Stock.DY);                                                           % Monthly log dividend yield
GER.Stock.Index  = data(:,3);                                                                   % Monthly stock index
GER.Stock.Ret    = log(GER.Stock.Index./lag(GER.Stock.Index,1,NaN)+GER.Stock.DY);
GER.Stock.ExRet  = GER.Stock.Ret - GER.RiskFree;
% -------------- Forex Exchanges ------------------------------------------

FX.EUvsUS=data(:,10);
FX.USvsUK=data(:,9);
FX.EUvsUK=FX.EUvsUS.*FX.USvsUK;
FX.r_EUvsUS  = log(data(:,10)./lag(data(:,10),1,NaN));
FX.r_USvsUK  = log((data(:,9))./lag(data(:,9),1,NaN));
FX.r_EUvsUK  = FX.r_EUvsUS + FX.r_USvsUK;

% -------------- US Stocks Monthly Excess Returns -------------------------

US.Stock.DY      = data(:,6)/(100*12);                                                          % US dividend yield
US.Stock.dy      = log(data(:,6));                                                              % US log dividend yield
US.Stock.Index   = data(:,5);                                                                   % US stock index
US.Stock.Ret     = log(US.Stock.Index./lag(US.Stock.Index,1,NaN)+US.Stock.DY);                        % Returns in dollar amounts
US.Stock.Reteu     = US.Stock.Ret + FX.r_EUvsUS;                                                    % Returns in euros (german perspective) 
US.Stock.ExRet   = US.Stock.Reteu - GER.RiskFree;                                                 % Excess returns in euros


% ------------- UK Stocks Monthly Excess Returns --------------------------

UK.Stock.DY      = data(:,8)/(100*12);                                                         % UK dividend yield
UK.Stock.dy      = log(data(:,8));                                                             % UK log dividend yield
UK.Stock.Index   = data(:,7);                                                                  % UK stock index
UK.Stock.Ret     = log(UK.Stock.Index./lag(UK.Stock.Index,1,NaN)+UK.Stock.DY);                  % Returns in dollar amounts
UK.Stock.Reteu     = UK.Stock.Ret + FX.r_EUvsUK;                                                    % Returns in euros (german perspective) 
UK.Stock.ExRet   = UK.Stock.Reteu - GER.RiskFree;                                                 % Excess returns in euros


%% ------------------------------------------------------------------------
% -------------------  Plot of the cumulative performance -----------------
% -------------------------------------------------------------------------

s_start   = '31/01/1978';
s_end     = '31/12/2003';
date_find = datenum([s_start; s_end],'dd/mm/yyyy');
ss        = datefind(date_find(1,1),date);
se        = datefind(date_find(2,1),date);
T=ss-se+1;
%-----------  Historical Excess Returns Matrix ----------------------------

R = [GER.LTBond.ExRet(ss:se) GER.Stock.ExRet(ss:se) UK.Stock.ExRet(ss:se) US.Stock.ExRet(ss:se)];

Perf = cumsum(R);
% Plot returns and cumulative returns performances  in figure(1)
run Figure001 

%% ------------------------------------------------------------------------
% ------------- Compute the weights and Efficient Frontier using information upto 2003
% -------------------------------------------------------------------------

% Compute unconditional means and VarCov
muR    = mean(R)';
SigmaR = (T/(T-1))*cov(R); %compute robust variance covariance matrix

% Optimal weights on the tangency portfolio

WeightsTP    = ((SigmaR^(-1))*muR)./(ones(size(R,2),1)'*(SigmaR^(-1)*muR));
%Compute the efficient frontier and plot it 
NumPortf = 20;  % number of efficient portfolios to be computed 
                % we do not compute all efficient portfolios in the
                % efficient frontier (there is a continuum of them)
                % we compute only a subset of them, equally spaced between
                % the minimum variance portfolio and the portfolio with the
                % highest return.

mu=mean(R)+ mean(GER.RiskFree(ss:se)); %generate mean returns
% generate mean variance efficient frontier
[WeightsMV, MuMeanVariance, StdMeanVariance] = effront(mu, SigmaR, NumPortf);
 
%mean variance for risk fre and tangency portfolio
Mu_bm = zeros(2, 1); 
SD_bm = zeros(2, 1);
Mu_bm(1,1) = mean(GER.RiskFree(ss:se));
SD_bm(1,1)=0;
Mu_bm(2,1)=mu*WeightsTP;
SD_bm(2,1)=sqrt(WeightsTP'*SigmaR*WeightsTP);
% Plot weights in the tangency portfolio and the efficient frontier
run Figure002
%% ------------------------------------------------------------------------
% Compute the cumulative performances out-of-sample
% -------------------------------------------------------------------------
p_start = '30/01/2004';
p_end   = '31/12/2007';
date_find=datenum([p_start; p_end],'dd/mm/yyyy');
ps=datefind(date_find(1,1),date);
pe=datefind(date_find(2,1),date);
n = pe-ps+1;

R2        = [GER.LTBond.ExRet(ps:pe) GER.Stock.ExRet(ps:pe) UK.Stock.ExRet(ps:pe) US.Stock.ExRet(ps:pe)];
PerfR2    = cumsum(R2);
Rport     = R2*WeightsTP;
PerfPort1 = cumsum(Rport);

% Plot the cumulative performances out-of-sample
run Figure003 

%% ------------------------------------------------------------------------
% ------------- Compute the weights and efficient frontier  using information from 2004 to 2007
% -------------------------------------------------------------------------

muR_end    = mean(R2)';
SigmaR_end = cov(R2);

% WEIGHTS 
WeightsTP_end = ((SigmaR_end^(-1))*muR_end)./(ones(4,1)'*(SigmaR_end^(-1)*muR_end));
% Performance 
Rport_end     = R2*WeightsTP_end;
PerfPort_end = cumsum(Rport_end);
%Efficient Frontier
NumPortf = 20;  % number of efficient portfolios to be computed 
                % we do not compute all efficient portfolios in the
                % efficient frontier (there is a continuum of them)
                % we compute only a subset of them, equally spaced between
                % the minimum variance portfolio and the portfolio with the
                % highest return.

mu_end=mean(R2)+ mean(GER.RiskFree(ps:pe)); %generate mean returns
% generate mean variance efficient frontier
[WeightsMV_end, MuMeanVariance_end, StdMeanVariance_end] = effront(mu_end, SigmaR_end, NumPortf);
%mean variance for risk fre and tangency portfolio
Mu_bm_end = zeros(2, 1); 
SD_bm_end = zeros(2, 1);
Mu_bm_end(1,1) = mean(GER.RiskFree(ps:pe));
SD_bm_end(1,1)=0;
Mu_bm_end(2,1)=mu_end*WeightsTP_end;
SD_bm_end(2,1)=sqrt(WeightsTP_end'*SigmaR_end*WeightsTP_end);
% Plot Weights, efficient frontier and comparative performance 
run Figure004

%% ------------------------------------------------------------------------
% ------------- Resampling Efficient Frontier and Tangency Portfolio
% -------------------------------------------------------------------------

%% Step 1. Construct the relevant time series of returns 
RetSeries = [GER.LTBond.Ret(ss:se) GER.Stock.Ret(ss:se) UK.Stock.Ret(ss:se) US.Stock.Ret(ss:se)];
ExRetSeries=[GER.LTBond.ExRet(ss:se) GER.Stock.ExRet(ss:se) UK.Stock.ExRet(ss:se) US.Stock.ExRet(ss:se)];
%% Step 2. Compute the Resampled Frontier
NumPortf = 20;
NumSimu =  1000;
[Wrsp,ERrsp,SDrsp,Wmv,MuMv,StdMv,Wmv_S,ERmv_S,SDmv_S] = resampfront(RetSeries, NumPortf, NumSimu,0);
[Wtp,Wtprsp,Wtprsp_S] = resamptp(ExRetSeries, NumSimu,0);
% Note:
% The outputs in the resampfront are the following:
% Wrsp  =  NumAssets time NumPortf matrix collecting the resampled portfolio weights
% ERrsp =  Column vector collecting resampled portfolio expected returns
% SRrsp =  Column vector collecting resampled portfolio standard deviations
% Wmv   =  Matrix collecting the mean-variance weights
% ERmv  =  Column vector collecting mean-variance portfolio expected returns
% SDmv  =  Column vector collectgin mean-variance portfolio standard deviations
% Wmv_S =  Array collecting the simulated mean-variance weights for statistical analysis purposes

% The outputs in the resamptp
% Wtp   = NumAssetx1 vector of weights on the TP.
% Wtprsp  = NumAsset*1 vector of mean TP weights in bootstrap.
% Wtprsp_S = NumAssets*1*NumSimu array collecting the simulated
%               weights (useful for statistical analysis).       

%% Step 3. Plots the statistical equivalence region and the resampled frontier

stateqregion(RetSeries, MuMv, StdMv,Wmv_S)
bsfrontier(MuMv,StdMv,MuMeanVariance_end, StdMeanVariance_end,ERmv_S,SDmv_S)

%% Step 4. Compare Performance of actual on BS Portfolio on smpl 2
Rport_bt     = R2*Wtprsp;
PerfPort_bt = cumsum(Rport_bt);
run Figure005
%% Step 5. Generates the output statistics 
PortfSet1=[1:20];
ConfLevel=5; % The confidence level 
[Stats] = Statistics(Wmv, Wmv_S, Wrsp, PortfSet1, ConfLevel); % Collect the simulations statistics
                                            

%% ------------------------------------------------------------------------
% ------------- Black-Litterman
% -------------------------------------------------------------------------

%Assumptions
wMP_i = WeightsTP;
lambda=(ones(size(R,2),1)'*(SigmaR^(-1)*muR));

% Implied expected excess returns
muMP_i = lambda*(SigmaR*wMP_i)+ mean(GER.RiskFree(ss:se));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Inputs, 

P = [ 0 1 0 0;
      0 0 1 0;
      0 0 0 1];

%lower bound 95% = 0, upper bound 95% = 0.08
V = [0.04/12 0.04/12 0.04/12]';
Gamma =[(0.02/12)^2  0     0    ;
        0     (0.02/12)^2  0    ;
        0     0     (0.02/12)^2];

tau = 0.3;

%B&L weights; 
K = tau*SigmaR*P'*(tau*P*SigmaR*P' + Gamma)^(-1);
muBL = muMP_i + K*(V-P*muMP_i);
lambdaBL=ones(size(R,2),1)'*(SigmaR^(-1)*(muBL-mean(GER.RiskFree(ss:se))));
wBL = SigmaR^(-1)*(muBL-mean(GER.RiskFree(ss:se)))/lambdaBL;
% Performance
RportBL = R2*wBL;
PerfPortBL = cumsum(RportBL);

% Plot, BL Portfolio Weights an Performance 
run Figure006

tElapsed = toc(TSTART);
display(['The time needed to run the code is: ',num2str(tElapsed/60),' minutes']);         










