function [Wtp,Wtprsp,Wtprsp_S] = resamptp(ExRetSeries, NumSimu,flag)
% RESAMPFRONT computes the resampled efficient frontier. Weights, expected
% returns and standard deviatios, both of resampled and mean variance
% efficient portfolios, are provided as outputs. Short sell constraint is 
% imposed.
%--------------------------------------------------------------------------
% INPUTS:  ExRetSeries = TxNumAssets matrix collecting asset returns.
%          NumSimu   = number of simulation to perform.
%          flag      = indicator variable to choose between MC simulation
%                      (flag =1) and non-parametric bootstrap (flag = 0)                      
%--------------------------------------------------------------------------
% OUTPUTS: Wtp   = NumAssetx1 vector of weights on the TP.
%          Wtprsp  = NumAsset*1 vector of meant TP weights in bootstrap.
%          Wtprsp_S = NumAssets*1*NumSimu array collecting the simulated
%               weights (useful for statistical analysis).       
%--------------------------------------------------------------------------           

T = size(ExRetSeries, 1); %determine time series length
NumAssets = size(ExRetSeries, 2); %determine number of assets
muExR = mean(ExRetSeries)'; %compute expected returns
SigmaExR = (T/(T-1))*cov(ExRetSeries); %compute variance covariance matrix

% generate TP weights (useful for comparison)
Wtp    = ((SigmaExR^(-1))*muExR)./(ones(NumAssets,1)'*(SigmaExR^(-1)*muExR));

% generate resampled efficient frontier
Wtprsp_S = zeros(NumAssets,  NumSimu); %preallocate weight matrix
if flag == 1
for i = 1:NumSimu
    ExRetSeries_S  = mvnrnd(muExR, SigmaExR,T); %simulate return series
    muExR_S   = mean(ExRetSeries_S)'; %compute expected returns
    SigmaExR_S     = (T/(T-1))*cov(ExRetSeries_S); %compute variance covariance matrix
    Wtprsp_S(:,i)=   ((SigmaExR_S^(-1))*muExR_S)./(ones(NumAssets,1)'*(SigmaExR_S^(-1)*muExR_S)); %generate tp weights
    display(num2str(i))
end
elseif flag ==0
for i = 1:NumSimu    
    k = randsample([1:T],T,'true',ones(T,1)/T);
    ExRetSeries_S = ExRetSeries(k,:);
    muExR_S   = mean(ExRetSeries_S)'; %compute expected returns
    SigmaExR_S     = (T/(T-1))*cov(ExRetSeries_S); %compute variance covariance matrix
    Wtprsp_S(:,i)=   ((SigmaExR_S^(-1))*muExR_S)./(ones(NumAssets,1)'*(SigmaExR_S^(-1)*muExR_S)); %generate tp weights
    display(num2str(i))
end  
end
Wtprsp = mean(Wtprsp_S,2); %average simulated tp weights


