clear

%We load the database
db=dlmread('zmdatabase.txt','\t');

%We save the number of information signals in our file
nv=18;

%We indicate the variables that we want to use and the number of stocks
%that we want to include in the portfolios
var=[1 7 9 16 4];

num=40;


%We convert all the -999 in the database into missing values
db(db==-999)=NaN;

%We load a file with the strings of the names of the Information Variables
run signals

%We split the database into its components
dat=datenum(db(:,1:3));
names=db(:,4);
dbx=db(:,5:end);

clear db


%We estract the list of the stocks, periods and dates
na=unique(names);
dates=unique(dat);

%We define the number of stocks and number of dates
ns=size(na,1);
nd=size(dates,1);

%We initialize vectors for portfolio returns
top=zeros(nd,size(var,2));
bot=top;
ben=top;




%We run a first outer loop on the base of the number of information signals
%chosen for the task
for i=1:size(var,2)
    
   
        
    %An inner loop will serve to repeat the ranking and invesmtment
    %decision each month
    for j=1:nd
        
        %We extract the sub-database with the observations that we
        %will use in this particular iteration
        dbx2=dbx(dat==dates(j),:);
        
        %We extract the sub-database where the value of the signal is
        %non-missing
        dbm=dbx2(isfinite(dbx2(:,var(i))),:);
        
        %We sort the rows of our sample on the base of he specific
        %information signal
        [dbs ind]=sortrows(dbm,-1*var(i));
        
        %We capture the average return (equally weighted portfolio) of the
        %stocks in the top and bottom positions of the sorted database
        top(j,i)=nanmean(dbs(1:num,nv+1));
        bot(j,i)=nanmean(dbs(end-num:end,nv+1));
        
        %We capture benchmark return in this month as the equally weighted
        %return of all stocks
        ben(j,i)=nanmean(dbs(1:end,nv+1));
    end
    
    %We save the name of the signal
    sn(1,i)=signal(var(i));
    
end



%We calculate some diagnostics
    
dris(1,:)=mean(top);
dris(2,:)=mean(bot);
dris(3,:)=mean(top)-mean(bot);
dris(4,:)=mean(top)-mean(ben);
dris(5,:)=mean(ben);

dris(6,:)=std(top);
dris(7,:)=std(bot);
dris(8,:)=std(top-bot);
dris(9,:)=std(top-ben);
dris(10,:)=std(ben);
    
ctop=corrcoef(top);


%We save the diagnostics to a xls file in an area prepared and formatted        
xlswrite('MM_Diagnostics.xls',sn,'Results','B1')
xlswrite('MM_Diagnostics.xls',dris,'Results','B2')

xlswrite('MM_Diagnostics.xls',sn,'Top','B1')
xlswrite('MM_Diagnostics.xls',[m2xdate(dates) top],'Top','A2')

xlswrite('MM_Diagnostics.xls',sn,'Bottom','B1')
xlswrite('MM_Diagnostics.xls',[m2xdate(dates) bot],'Bottom','A2')

xlswrite('MM_Diagnostics.xls',sn,'Correlations','B1')
xlswrite('MM_Diagnostics.xls',sn','Correlations','A2')
xlswrite('MM_Diagnostics.xls',ctop,'Correlations','B2')
