%We load the database
db=dlmread('zmdatabase.txt','\t');

%We load the top portfolio from the previous analysis
dbt=xlsread('MM_Diagnostics','Top','B2:E250');

%We load the name of the information variables
run signals

%We save the number of information signals in our file
nv=18;

%We indicate the variables that we used in the model
var=[1 7 9 16];

%We calculate mean return and risk for the top portfolios
[mrt covt]=ewstats(dbt);

%We calculate the efficient frontier for the top portfolios
[ri re w]=frontcon(mrt,covt,10);

%We calculate the Reward to Risk Ratio for the frontier portfolios
rrr=re./ri;

%We look for the portoflio that maximizes the Reward to Risk Ratio
x=find(rrr==max(rrr));

%We indicate the weigths for the model
we=w(x,:);

num=40;

%We convert all the -999 in the database into missing values
db(db==-999)=NaN;

%We load a file with the strings of the names of the Information Variables
run signals

%We split the database into its components
dat=datenum(db(:,1:3));
names=db(:,4);
dbx=db(:,5:end);

clear db


%We estract the list of the stocks, periods and dates
na=unique(names);
dates=unique(dat);

%We define the number of stocks and number of dates
ns=size(na,1);
nd=size(dates,1);

%We initialize vectors for portfolio returns
top=zeros(nd,1);
bot=top;
ben=top;

%We create the model using the optimal weights
    
mod=dbx(:,var)*we';

%For simplicity we substitute the model to the first screening variable

dbx(:,1)=mod;


%We repeat the ranking and invesmtment decision each month

for j=1:nd-1

    %We extract the sub-database with the observations that we
    %will use in this particular iteration
     dbx2=dbx(dat==dates(j),:);
   
    %We extract the sub-database where the value of the signal is
    %non-missing
    dbm=dbx2(isfinite(dbx2(:,1)),:);

    %We sort the rows of our sample on the base of he specific
    %information signal
    [dbs ind]=sortrows(dbm,-1*var(1));

    %We capture the average return (equally weighted portfolio) of the
    %stocks in the top and bottom positions of the sorted database
    top(j,1)=nanmean(dbs(1:num,nv+1));
    bot(j,1)=nanmean(dbs(end-num:end,nv+1));

    %We capture benchmark return in this month as the equally weighted
    %return of all stocks
    ben(j,1)=nanmean(dbs(1:end,nv+1));
end


%We calculate some diagnostics
    
dris(1,:)=mean(top);
dris(2,:)=mean(bot);
dris(3,:)=mean(top)-mean(bot);
dris(4,:)=mean(top)-mean(ben);
dris(5,:)=mean(ben);

dris(6,:)=std(top);
dris(7,:)=std(bot);
dris(8,:)=std(top-bot);
dris(9,:)=std(top-ben);
dris(10,:)=std(ben);

ptop=ret2tick(top);
pbot=ret2tick(bot);
pben=ret2tick(ben);

%We plot the results

datex=[dates(1)-30;dates];

figure
plot(datex,[ptop pbot pben])
legend('Top','Bottom','Benchmark')
grid on
dateaxis('x',12)
title('Z-Score Model')
    
    
%We save the diagnostics to a xls file in an area prepared and formatted
xlswrite('MM_Diagnostics.xls',dris,'Results','C16')